@@ -45,26 +45,26 @@
 #
 
 # Locate the global Python where virtualenvwrapper is installed.
-if [ "$VIRTUALENVWRAPPER_PYTHON" = "" ]
+if [ "${VIRTUALENVWRAPPER_PYTHON:-}" = "" ]
 then
     VIRTUALENVWRAPPER_PYTHON="$(command \which python)"
 fi
 
 # Set the name of the virtualenv app to use.
-if [ "$VIRTUALENVWRAPPER_VIRTUALENV" = "" ]
+if [ "${VIRTUALENVWRAPPER_VIRTUALENV:-}" = "" ]
 then
     VIRTUALENVWRAPPER_VIRTUALENV="virtualenv"
 fi
 
 # Set the name of the virtualenv-clone app to use.
-if [ "$VIRTUALENVWRAPPER_VIRTUALENV_CLONE" = "" ]
+if [ "${VIRTUALENVWRAPPER_VIRTUALENV_CLONE:-}" = "" ]
 then
     VIRTUALENVWRAPPER_VIRTUALENV_CLONE="virtualenv-clone"
 fi
 
 # Define script folder depending on the platorm (Win32/Unix)
 VIRTUALENVWRAPPER_ENV_BIN_DIR="bin"
-if [ "$OS" = "Windows_NT" ] && ([ "$MSYSTEM" = "MINGW32" ] || [ "$MSYSTEM" = "MINGW64" ])
+if [ "${OS:-}" = "Windows_NT" ] && ([ "${MSYSTEM:-}" = "MINGW32" ] || [ "${MSYSTEM:-}" = "MINGW64" ])
 then
     # Only assign this for msys, cygwin use standard Unix paths
     # and its own python installation
@@ -73,7 +73,7 @@ fi
 
 # Let the user override the name of the file that holds the project
 # directory name.
-if [ "$VIRTUALENVWRAPPER_PROJECT_FILENAME" = "" ]
+if [ "${VIRTUALENVWRAPPER_PROJECT_FILENAME:-}" = "" ]
 then
     export VIRTUALENVWRAPPER_PROJECT_FILENAME=".project"
 fi
@@ -83,7 +83,7 @@ fi
 export VIRTUALENVWRAPPER_WORKON_CD=${VIRTUALENVWRAPPER_WORKON_CD:-1}
 
 # Remember where we are running from.
-if [ -z "$VIRTUALENVWRAPPER_SCRIPT" ]
+if [ -z "${VIRTUALENVWRAPPER_SCRIPT:-}" ]
 then
     if [ -n "$BASH" ]
     then
@@ -108,10 +108,10 @@ fi
 # we are trying to change the state of the current shell, so we use
 # "builtin" for bash and zsh but "command" under ksh.
 function virtualenvwrapper_cd {
-    if [ -n "$BASH" ]
+    if [ -n "${BASH:-}" ]
     then
         builtin \cd "$@"
-    elif [ -n "$ZSH_VERSION" ]
+    elif [ -n "${ZSH_VERSION:-}" ]
     then
         builtin \cd -q "$@"
     else
@@ -228,7 +228,7 @@ function virtualenvwrapper_run_hook {
     ( \
         virtualenvwrapper_cd "$WORKON_HOME" &&
         "$VIRTUALENVWRAPPER_PYTHON" -m 'virtualenvwrapper.hook_loader' \
-            $HOOK_VERBOSE_OPTION --script "$hook_script" "$@" \
+            ${HOOK_VERBOSE_OPTION:-} --script "$hook_script" "$@" \
     )
     result=$?
 
@@ -244,8 +244,8 @@ function virtualenvwrapper_run_hook {
         source "$hook_script"
     elif [ "${1}" = "initialize" ]
     then
-        cat - 1>&2 <<EOF 
-virtualenvwrapper.sh: There was a problem running the initialization hooks. 
+        cat - 1>&2 <<EOF
+virtualenvwrapper.sh: There was a problem running the initialization hooks.
 
 If Python could not import the module virtualenvwrapper.hook_loader,
 check that virtualenvwrapper has been installed for
@@ -260,7 +260,7 @@ EOF
 # Set up tab completion.  (Adapted from Arthur Koziel's version at
 # http://arthurkoziel.com/2008/10/11/virtualenvwrapper-bash-completion/)
 function virtualenvwrapper_setup_tab_completion {
-    if [ -n "$BASH" ] ; then
+    if [ -n "${BASH:-}" ] ; then
         _virtualenvs () {
             local cur="${COMP_WORDS[COMP_CWORD]}"
             COMPREPLY=( $(compgen -W "`virtualenvwrapper_show_workon_options`" -- ${cur}) )
@@ -304,8 +304,9 @@ function virtualenvwrapper_initialize {
     # Set the location of the hook scripts
     if [ "$VIRTUALENVWRAPPER_HOOK_DIR" = "" ]
     then
-        export VIRTUALENVWRAPPER_HOOK_DIR="$WORKON_HOME"
+        VIRTUALENVWRAPPER_HOOK_DIR="$WORKON_HOME"
     fi
+    export VIRTUALENVWRAPPER_HOOK_DIR
 
     mkdir -p "$VIRTUALENVWRAPPER_HOOK_DIR"
 
@@ -515,7 +516,7 @@ function rmvirtualenv {
     virtualenvwrapper_verify_workon_home || return 1
     if [ ${#@} = 0 ]
     then
-        echo "Please specify an enviroment." >&2
+        echo "Please specify an environment." >&2
         return 1
     fi
 
@@ -557,7 +558,7 @@ function rmvirtualenv {
 # List the available environments.
 function virtualenvwrapper_show_workon_options {
     virtualenvwrapper_verify_workon_home || return 1
-    # NOTE: DO NOT use ls or cd here because colorized versions spew control 
+    # NOTE: DO NOT use ls or cd here because colorized versions spew control
     #       characters into the output list.
     # echo seems a little faster than find, even with -depth 3.
     # Note that this is a little tricky, as there may be spaces in the path.
@@ -572,7 +573,7 @@ function virtualenvwrapper_show_workon_options {
     #    a slash, as that is an illegal character in a directory name.
     #    This yields a slash-separated list of possible env names.
     # 4. Replace each slash with a newline to show the output one name per line.
-    # 5. Eliminate any lines with * on them because that means there 
+    # 5. Eliminate any lines with * on them because that means there
     #    were no envs.
     (virtualenvwrapper_cd "$WORKON_HOME" && echo */$VIRTUALENVWRAPPER_ENV_BIN_DIR/activate) 2>/dev/null \
         | command \tr "\n" " " \
@@ -593,7 +594,7 @@ function _lsvirtualenv_usage {
 function lsvirtualenv {
 
     typeset long_mode=true
-    if command -v "getopts" &> /dev/null
+    if command -v "getopts" >/dev/null 2>&1
     then
         # Use getopts when possible
         OPTIND=1
@@ -749,12 +750,17 @@ function workon {
 
     # Deactivate any current environment "destructively"
     # before switching so we use our override function,
-    # if it exists.
+    # if it exists, but make sure it's the deactivate function
+    # we set up
     type deactivate >/dev/null 2>&1
     if [ $? -eq 0 ]
     then
-        deactivate
-        unset -f deactivate >/dev/null 2>&1
+        type deactivate | grep 'typeset env_postdeactivate_hook' >/dev/null 2>&1
+        if [ $? -eq 0 ]
+        then
+            deactivate
+            unset -f deactivate >/dev/null 2>&1
+        fi
     fi
 
     virtualenvwrapper_run_hook "pre_activate" "$env_name"
@@ -809,7 +815,7 @@ function virtualenvwrapper_get_python_version {
 
 # Prints the path to the site-packages directory for the current environment.
 function virtualenvwrapper_get_site_packages_dir {
-    "$VIRTUAL_ENV/$VIRTUALENVWRAPPER_ENV_BIN_DIR/python" -c "import distutils; print(distutils.sysconfig.get_python_lib())"
+    "$VIRTUAL_ENV/$VIRTUALENVWRAPPER_ENV_BIN_DIR/python" -c "import distutils.sysconfig; print(distutils.sysconfig.get_python_lib())"
 }
 
 # Path management for packages outside of the virtual env.
@@ -947,7 +953,7 @@ function cpvirtualenv {
     typeset src_name="$1"
     typeset trg_name="$2"
     typeset src
-    typeset trg 
+    typeset trg
 
     # without a source there is nothing to do
     if [ "$src_name" = "" ]; then
@@ -991,10 +997,10 @@ function cpvirtualenv {
 
     echo "Copying $src_name as $trg_name..."
     (
-        [ -n "$ZSH_VERSION" ] && setopt SH_WORD_SPLIT 
+        [ -n "$ZSH_VERSION" ] && setopt SH_WORD_SPLIT
         virtualenvwrapper_cd "$WORKON_HOME" &&
-        "$VIRTUALENVWRAPPER_VIRTUALENV_CLONE" "$src" "$trg" 
-        [ -d "$trg" ] && 
+        "$VIRTUALENVWRAPPER_VIRTUALENV_CLONE" "$src" "$trg"
+        [ -d "$trg" ] &&
             virtualenvwrapper_run_hook "pre_cpvirtualenv" "$src" "$trg_name" &&
             virtualenvwrapper_run_hook "pre_mkvirtualenv" "$trg_name"
     )
@@ -1281,7 +1287,7 @@ function wipeenv {
     virtualenvwrapper_verify_active_environment || return 1
 
     typeset req_file="$(virtualenvwrapper_tempfile "requirements.txt")"
-    pip freeze | egrep -v '(distribute|wsgiref)' > "$req_file"
+    pip freeze | egrep -v '(distribute|wsgiref|appdirs|packaging|pyparsing|six)' > "$req_file"
     if [ -n "$(cat "$req_file")" ]
     then
         echo "Uninstalling packages:"
